function MCdistance_ctrl (sigma, T, M)
% La fonction 'MCdistance_ctrl' fait le même travail que 'MCdistance', mais
% en utilisant les variables de contrôle suggérées par l'énoncé. Notez que
% puisqu'ici la matrice de covariance par unité de temps C doit être
% isotrope, on la passe en argument sous la forme du scalaire 'sigma' tel
% que C = sigma^2*I_2.

% Outre les sommes habituelles 'S' et 'S2', ici il va falloir tenir à jour
% la somme SZ des variables de contrôle (Z_2,Z_1) ainsi que la somme SZ2 de
% leurs “carrés” (incluant les produits croisés), et aussi la somme SP des
% produits entre les variables aléatoires distance et (Z_2,Z_1).
S = 0;
S2 = 0;
SZ = zeros(1,2);
SZ2 = zeros(2,2);
SP = zeros(1,2);

for j = 1:M
    % Dans cette version la sous-routine de simulation, que j'appelle ici
    % 'simulation_prMC_c', renvoie non seulement la distance parcourue à
    % vol d'oiseau mais aussi la valeur des variables de contrôle.
    [distance,Z2,Z1] = simulation_prMC_c(sigma,T);
    % On maintient S et S2, mais aussi SZ, SZ2 et SP.
    S = S + distance;
    S2 = S2 + distance * distance;
    SZ = SZ + [Z2,Z1];
    SZ2 = SZ2 + [Z2,Z1]' * [Z2,Z1];
    SP = SP + distance * [Z2,Z1];
end

% Dans un premier temps, on en déduit (une estimation de) la covariance
% 'CovCtrl' entre distance et (Z_2,Z_1), ainsi que de la matrice de
% covariance 'VarCtrl' de (Z2,Z1). Notez que cela impose au passage de
% calculer les moyennes empiriques respectives 'm' et 'mCtrl' de distance
% et (Z_2,Z_1).
m = S / M;
mCtrl = SZ / M;
CovCtrl = SP / M - m * mCtrl;
VarCtrl = SZ2 / M - mCtrl' * mCtrl;

% On en déduit (une estimation de) le vecteur optimal lambda tel que
% distance-lambda*(Z_2,Z_1)' ait une variance minimale, à l'aide de la
% formule du cours. Rappelons que sous Matlab, si 'V' est un vecteur-ligne
% et 'M' une matrice carrée de même dimension, « V / M » signifie « VM^{-1}
% » (*1).
lambda = CovCtrl / VarCtrl;

% Maintenant, il faut savoir ce qu'on aurait trouvé comme somme et somme
% des carrés si on avait utilisé la “bonne” variable, à savoir
% distance-lambda*(Z_2,Z_1)'. J'appelle 'Sbon' et 'S2bon' les quantités en
% question. Ce qui est remarquable avec la méthode des variables de
% contrôle, c'est qu'il n'y a rien à recalculer: on peut déduire Sbon et
% S2bon des sommes maintenues dans notre boucle!
Sbon = S - lambda * SZ';
S2bon = S2 - 2 * lambda * SP' + lambda * SZ2 * lambda';
% On en déduit l'intervalle de confiance sur la “bonne” variable.
mbon = Sbon / M;
vbon = S2bon / M - mbon * mbon;
margebon = 1.96 * sqrt (vbon / M);
% Comme on connait exactement l'espérance 'EZ' de (Z_2,Z_1), on en déduit
% enfin l'intervalle de confiance sur la variable distance.
EZ = [T^2*sigma^2, sigma*T^(3/2)*sqrt(2*pi)/3];
estimateurvrai = mbon + lambda * EZ';
margevrai = margebon;
fprintf('Distance à vol d''oiseau moyenne %.4f ± %.4f\n', ...
    estimateurvrai, margevrai);
return



% Sous-routine 'simulation_prMC_c'
    function [distance, Z2, Z1] = simulation_prMC_c (sigma, T)
        % La fonction 'simulation_prMC_c' effectue le même travail que
        % 'simulation_prMC', à ceci près qu'elle retourne aussi les valeurs
        % 'Z2' et 'Z1' des variables de contrôle respectives Z_2 et Z_1 ;
        % et qu'elle prend en argument sigma et non plus R pour le bruit
        % brownien, vu qu'ici ce bruit est isotrope.
        N = 1600;
        dt = T / N;
        X = zeros(2,N+1);
        Y = zeros(2,N+1);
        X(:,1) = [0;0];
        Y(:,1) = [0;0];
        for i = 1:N
            X(:,i+1) = X(:,i) + Y(:,i) * dt;
            % Pour simuler l'incrément d'un mouvement brownien isotrope,
            % notez que « randn(2,1) » simule un vecteur isotrope de
            % covariance I_2, de sorte qu'il n'y a plus qu'à multiplier par
            % le scalaire sigma.
            Y(:,i+1) = Y(:,i) + sigma * randn(2,1) * sqrt(dt);
        end
        distance = sqrt( sum((X(:,N+1)-X(:,1)).^2));
        
        % On calcule mainteant Z2 et Z1. Bien entendu, on n'a pas accès à
        % la valeur exacte de ces intégrales, mais seulement à une version
        % discrétisée. J'utilise ici la méthode d'intégration des trapèzes,
        % ce qui explique que les points extrêmaux de l'intervalle de temps
        % soient affectés chacun d'un coefficient 1/2. On commence pour
        % cela par calculer les vecteurs "vitesses1" et "vitesses2" des
        % normes des vitesses et de leurs carrés. Notez que la fonction
        % 'sum' de Matlab, quand on l'applique à un matrice non-ligne,
        % calcule le vecteur-ligne des sommes de chaque colonne. (*2)
        vitesses2 = sum(Y.^2);
        vitesses1 = sqrt(vitesses2);
        Z2 = dt * (vitesses2(1) / 2 + sum(vitesses2(2:N)) + ...
            vitesses2(N+1) / 2);
        Z1 = dt * (vitesses1(1) / 2 + sum(vitesses1(2:N)) + ...
            vitesses1(N+1) / 2);
        return
    end

end


% (*1) Le sens de « V / M » est en fait un tout petit peu plus général, car
% y a aussi une signification dans certains cas où M n'est pas inversible.

% (*2) Pour forcer ce comportement également dans le cas d'une
% matrice-ligne 'L', il faut préciser « sum(L,1) » (le « 1 » signifie qu'on
% somme L selon la première direction, à savoir colonne par colonne).