function simulation (C, T)
% La fonction 'simulation' simule le système d'équations différentielles
% (1) et trace la trajectoire suivie par X dans R^2. 'C' est la matrice de
% covariance par unité de temps du bruit brownien auquel est soumise la
% vitesse, et 'T' est la durée de temps sur laquelle on effectue la
% simulation.

% 'N' est le nombre de pas de temps pour la discrétisation.
N = 1600;
% 'dt' est l'intervalle de temps entre deux instants de discrétisation.
dt = T / N;
% Allocation de mémoire pour (X_t)_t et pour (Y_t)_t. Noter que X (resp. Y)
% est une fonction à valeurs dans R^2, de sorte que ses valeurs successives
% seront stockées dans une matrice de 2 lignes appelée 'X' (resp. Y).
% Attention, la colonne i de la matrice des positions (ou des vitesses) ne
% correspondra pas au temps i*dt mais à (i-1)*dt, puisque l'indexation des
% matrices de Matlab commence à 1.
X = zeros(2,N+1);
Y = zeros(2,N+1);
% On calcule la matrice de Cholesky de C une fois pour toutes, et on
% l'appelle 'R'.
R = chol(C);

% Initialisation de X et Y au temps 0. (Ici cette étape est en fait
% superflue vu que X et Y partent de valeurs nulles).
X(:,1) = [0;0];
Y(:,1) = [0;0];
% Boucle de simulation par la méthode d'Euler stochastique. Le fait qu'on
% ait ici un système d'équations ne change rien au principe. Attention, ici
% le bruit brownien correspond à un vecteur /colonne/ ; il faut donc
% transposer la méthode Cholesky. On prendra garde à ce que la matrice de
% Cholesky correspondant à dt*C est sqrt(dt)*R : il en effet une racine
% carrée pour dt dans la mesure où la matrice de base est quadratique en la
% matrice de Cholesky.
for i = 1:N
    X(:,i+1) = X(:,i) + Y(:,i) * dt;
    Y(:,i+1) = Y(:,i) + R' * randn(2,1) * sqrt(dt);
end

% On trace le résultat : la trajectoire suivie par X n'est autre que le
% graphe de la deuxième ligne (positions en y) de la matrice X contre sa
% première ligne (positions en x). Noter l'utilisation de « axis equal »
% pour avoir la même échelle sur les deux axes.
plot (X(1,:), X(2,:));
axis equal;
return

end